/*
 
 Jérôme Suhard, jerome@suhard.fr
 
 $ cmake -DGeant4_DIR=~/dmg_gate/install/10.5/geant4/lib/Geant4-9.6.0/ .
 
 usage:
 $ export G4LEDATA="/path/to/G4EMLOW6.23"
 $ ./example01 5000
 
 */

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4PhysListFactory.hh"
#include "G4EmDNAPhysics.hh"
#ifdef G4UI_USE
    #include "G4UIExecutive.hh"
    #ifdef G4VIS_USE
        #include "G4VisExecutive.hh"
    #endif
#endif

#include "ExN01DetectorConstruction.hh"
#include "ExN01PhysicsList.hh"
#include "ExN01PrimaryGeneratorAction.hh"

class steppingAction : public G4UserSteppingAction {
public:
    steppingAction() {
        sum_edep.resize(600);
    };
    void UserSteppingAction(const G4Step * step) {
        const G4String name = step->GetTrack()->GetVolume()->GetName();
        if(name=="world")   return;
        const double weight = step->GetTrack()->GetWeight();
        const double edep = step->GetTotalEnergyDeposit()*weight;
        if(edep > 0 && std::isfinite(edep)) {
            const G4ThreeVector prePosition = step->GetPreStepPoint()->GetPosition();
            const G4ThreeVector postPosition = step->GetPostStepPoint()->GetPosition();
            const G4double x = G4UniformRand();
            const G4ThreeVector direction = postPosition - prePosition;
            const G4ThreeVector position = prePosition + x * direction;
            const G4int index = getIndexFromPosition(position);
            sum_edep[index] += edep;
            //G4cout << "[" << name << "] " << prePosition << " -> " << postPosition << " : " << position << " " << index << " " << edep << " " << sum_edep.at(index) << G4endl;
        }
        
    }
    G4int getIndexFromPosition(const G4ThreeVector v) {
        G4double x = v.getX()*cm + 30*cm;
        return G4int (x * 600.0 / (60*cm));
    }
    void print(std::ostream & file) {
        for(int i = 0; i < 600; i++) {
            file << sum_edep[i]*MeV << "\t";
        }
        file << G4endl;
    }
private:
    std::vector<double> sum_edep;
};

int main(int argc, char** argv)
{

  G4RunManager* runManager = new G4RunManager;

  runManager->SetUserInitialization(new ExN01DetectorConstruction);
  
  G4VUserPhysicsList* physics;
    physics = new ExN01PhysicsList;
    /*
    G4PhysListFactory * physFactory = new G4PhysListFactory;
    physics = physFactory->GetReferencePhysList("FTFP_BERT_LIV");
    delete physFactory;
    */
     
  runManager->SetUserInitialization(physics);
    
  runManager->SetUserAction(new ExN01PrimaryGeneratorAction);
    
  steppingAction * action = new steppingAction;
  runManager->SetUserAction(action);

  // Initialize G4 kernel
  //
  runManager->Initialize();

  // Get the pointer to the UI manager and set verbosities
  //
  G4UImanager* UI = G4UImanager::GetUIpointer();
  UI->ApplyCommand("/run/verbose 2");
  //UI->ApplyCommand("/event/verbose 1");
  //UI->ApplyCommand("/tracking/verbose 1");

    bool isGUI = false;
    #ifdef G4UI_USE
        if(argc == 1)   isGUI = true;
    #endif
    
    if(isGUI) {
        #ifdef G4VIS_USE
            G4VisManager* visManager = new G4VisExecutive;
            visManager->SetVerboseLevel(G4VisManager::quiet);
            visManager->Initialize();
        #endif
        G4UIExecutive * ui = new G4UIExecutive( argc, argv );
        #ifdef G4VIS_USE
            UI->ApplyCommand("/vis/open OGL");
            UI->ApplyCommand("/vis/drawVolume");
            UI->ApplyCommand("/vis/scene/add/axes 0 0 0 10 cm");
            UI->ApplyCommand("/vis/scene/add/trajectories");
            UI->ApplyCommand("/vis/scene/endOfEventAction accumulate");
        #endif
        ui->SessionStart();
        delete ui;
        #ifdef G4VIS_USE
            delete visManager;
        #endif
    }else {
        const G4int numberOfEvent = atoi(argv[1]);
        runManager->BeamOn(numberOfEvent);
    }

    std::ofstream file("test.txt");
    action->print(file);
    file.close();
    
  delete runManager;

  return 0;
}


